import IWebSocket from "./socket";
import * as WebSocket from "ws";

export class NodeWebSocket implements IWebSocket {
   ws: WebSocket;
   constructor(url: string) {
      this.ws = new WebSocket(url);
   }

   onopen(callback: () => void): void {
      this.ws.on("open", callback);
   }
   onmessage(callback: (data: string) => void): void {
      this.ws.on("message", (data) => {
         if (data instanceof ArrayBuffer) {
            data = new TextDecoder().decode(data);
         } else if (data instanceof Buffer) {
            data = data.toString("utf-8");
         } else if (Array.isArray(data)) {
            data = Buffer.concat(data).toString("utf-8");
         } else if (typeof data !== "string") {
            throw new Error("Unknown data encoding");
         }
         callback(data);
      });
   }
   onclose(callback: () => void): void {
      this.ws.on("close", callback);
   }
   onerror(callback: (err: Error) => void): void {
      this.ws.on("error", callback);
   }

   send(data: string): void {
      this.ws.send(data);
   }

   close(): void {
      this.ws.close();
   }
}

import Database, { WebSocketConnector } from "./database";
WebSocketConnector.WS = NodeWebSocket;

export { Database };
export default Database;

export * from "./index";
